package org.jboss.qa.brms;

import org.junit.Assert;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import java.net.MalformedURLException;
import java.net.URL;
import org.kie.internal.runtime.conf.RuntimeStrategy;
import org.kie.services.client.serialization.jaxb.impl.deploy.JaxbDeploymentUnit;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Please edit /test-config.properties as needed by your environment.
 */
public class ClusterSetupTest {

    private static final Logger LOGGER = LoggerFactory.getLogger(ClusterSetupTest.class);

    private static final int TIMEOUT = 120 * 1000; //msec
    private static final int WAIT_FOR_REPOSITORY_INDEXING = 30 * 1000;
    private static final String BPMS_ASSETS = "git://git.app.eng.bos.redhat.com/bpms-assets.git";
    private static final String REPO_NAME = "bpms-assets";
    private static final String PROJECT = "clustering";
    private static final String DEPLOYMENT_UNIT = "org.jboss.qa.bpms:clustering:1.0.0.Final";
    private static final String SESSION_STRATEGY = "PER_PROCESS_INSTANCE";
    private static final String ORG_UNIT = "test";

    private final String host = TestParameters.get(TestParameters.PARAM_HOST);
    private final int portBase = Integer.parseInt(TestParameters.get(TestParameters.PARAM_PORT_BASE));
    private final int portOffset = Integer.parseInt(TestParameters.get(TestParameters.PARAM_PORT_OFFSET));
    private final String context = TestParameters.get(TestParameters.PARAM_CTX);
    private final String user = TestParameters.get(TestParameters.PARAM_USER);
    private final String password = TestParameters.get(TestParameters.PARAM_PASSWORD);

    private WorkbenchClient workbenchClient;

    public ClusterSetupTest() {
        URL url = getUrl(0);
        workbenchClient = new WorkbenchClientWithAcknowledgement(RestWorkbenchClient.createInstance
                (url.toExternalForm(), user, password));
        LOGGER.info("Using URL " + url.toExternalForm());
    }

    @Before
    public void cloneRepository() throws NotSuccessException {
        workbenchClient.createOrganizationalUnit(ORG_UNIT, "");
        workbenchClient.createRepository(REPO_NAME, BPMS_ASSETS, ORG_UNIT);

        try { //waiting for the repository to be indexed, otherwise the project won't be visible in further steps
            Thread.sleep(WAIT_FOR_REPOSITORY_INDEXING);
        } catch (InterruptedException e) {
            LOGGER.error("Interrupted in waiting.", e);
        }
    }

    @Test(timeout = TIMEOUT)
    public void testInstallUndeployDeploy() throws NotSuccessException {
        //install - default deployment unit should be created
        workbenchClient.installProject(REPO_NAME, PROJECT);
        JaxbDeploymentUnit jaxbDeploymentUnit = workbenchClient.getDeploymentUnit(DEPLOYMENT_UNIT);
        Assert.assertEquals(JaxbDeploymentUnit.JaxbDeploymentStatus.DEPLOYED, jaxbDeploymentUnit.getStatus());

        //...so let's drop it
        workbenchClient.removeDeploymentUnit(DEPLOYMENT_UNIT);
        jaxbDeploymentUnit = workbenchClient.getDeploymentUnit(DEPLOYMENT_UNIT);
        Assert.assertEquals(JaxbDeploymentUnit.JaxbDeploymentStatus.UNDEPLOYED, jaxbDeploymentUnit.getStatus());

        //...and re-create
        workbenchClient.createDeploymentUnit(DEPLOYMENT_UNIT, SESSION_STRATEGY);
        jaxbDeploymentUnit = workbenchClient.getDeploymentUnit(DEPLOYMENT_UNIT);
        Assert.assertEquals(JaxbDeploymentUnit.JaxbDeploymentStatus.DEPLOYED, jaxbDeploymentUnit.getStatus());
        Assert.assertEquals(RuntimeStrategy.PER_PROCESS_INSTANCE, jaxbDeploymentUnit.getStrategy());
    }

    private URL getUrl(final int index) {
        final String urlString = String.format("http://%s:%d/%s", host, portBase + index * portOffset, context);
        try {
            return new URL(urlString);
        } catch (MalformedURLException ex) {
            throw new RuntimeException(ex);
        }
    }
}
